@@ -1,7 +1,7 @@
 /*@z04.c:Token Service:NewToken(), CopyTokenList()@***************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.24)                       */
-/*  COPYRIGHT (C) 1991, 2000 Jeffrey H. Kingston                             */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.25)                       */
+/*  COPYRIGHT (C) 1991, 2001 Jeffrey H. Kingston                             */
 /*                                                                           */
 /*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
@@ -218,12 +218,15 @@ FULL_CHAR *EchoToken(OBJECT x)
     case GRAPHIC:
     case LINK_SOURCE:
     case LINK_DEST:
+    case LINK_URL:
     case ACAT:
     case HCAT:
     case VCAT:
     case CLOSURE:
     case PREPEND:
     case SYS_PREPEND:
+    case INCG_REPEATED:
+    case SINCG_REPEATED:
     case DATABASE:
     case SYS_DATABASE:
     case LUSE:
