@@ -1,7 +1,7 @@
 /*@z44.c:Vertical Hyphenation:VerticalHyphenate()@****************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.24)                       */
-/*  COPYRIGHT (C) 1991, 2000 Jeffrey H. Kingston                             */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.25)                       */
+/*  COPYRIGHT (C) 1991, 2001 Jeffrey H. Kingston                             */
 /*                                                                           */
 /*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
@@ -553,7 +553,7 @@ OBJECT BuildEnclose(OBJECT hd)
   FposCopy(fpos(par), fpos(hd));
   actual(par) = parsym;
   Link(x, par);
-  val = MakeWord(WORD, AsciiToFull("??"), &fpos(hd));
+  val = MakeWord(WORD, AsciiToFull("???"), &fpos(hd));
   Link(par, val);
 
   /* set env to the appropriate environment for this symbol */
